@@ -1,7 +1,7 @@
-/* Copyright © 2007-2015 Jakub Wilk <jwilk@jwilk.net>
+/* Copyright © 2007-2017 Jakub Wilk <jwilk@jwilk.net>
  * Copyright © 2009 Mateusz Turcza
  *
- * This file is part of pdfdjvu.
+ * This file is part of pdf2djvu.
  *
  * pdf2djvu is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
@@ -26,6 +26,7 @@
 #include <iostream>
 #include <sstream>
 #include <stdexcept>
+#include <vector>
 
 #include <dirent.h>
 #include <libgen.h>
@@ -37,7 +38,6 @@
 #include <windows.h>
 #endif
 
-#include "array.hh"
 #include "debug.hh"
 #include "i18n.hh"
 #include "string-printf.hh"
@@ -221,12 +221,12 @@ std::string Win32Error::error_message(const std::string &context)
   unsigned long error_code = GetLastError();
   unsigned long nbytes = FormatMessage(
     FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
-    NULL,
+    nullptr,
     error_code,
     MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
     (char*) &buffer,
     0,
-    NULL
+    nullptr
   );
   if (nbytes == 0)
     message.append(_("possibly memory allocation error"));
@@ -251,7 +251,7 @@ void throw_win32_error(const std::string &context)
  */
 
 Directory::Directory(const std::string &name)
-: name(name), posix_dir(NULL)
+: name(name), posix_dir(nullptr)
 {
   this->open(name.c_str());
 }
@@ -264,39 +264,50 @@ Directory::~Directory() throw ()
 void Directory::open(const char* path)
 {
   this->posix_dir = opendir(path);
-  if (this->posix_dir == NULL)
+  if (this->posix_dir == nullptr)
     throw_posix_error(path);
 }
 
-void Directory::close(void)
+void Directory::close()
 {
-  if (this->posix_dir == NULL)
+  if (this->posix_dir == nullptr)
     return;
   if (closedir(static_cast<DIR*>(this->posix_dir)) != 0)
     throw_posix_error(this->name);
 }
 
 
-/* class TemporaryPathTemplate : Array<char>
- * =========================================
+/* class TemporaryPathTemplate
+ * ===========================
  */
 
 
-class TemporaryPathTemplate : public Array<char>
+class TemporaryPathTemplate
 {
 protected:
+  std::vector<char> buffer;
   static const char *temporary_directory()
   {
     const char *result = getenv("TMPDIR");
-    if (result == NULL)
+    if (result == nullptr)
       result = P_tmpdir;
     return result;
   }
 public:
   TemporaryPathTemplate()
-  : Array<char>(strlen(this->temporary_directory()) + strlen(PACKAGE_NAME) + 9)
+  : buffer(strlen(this->temporary_directory()) + strlen(PACKAGE_NAME) + 9)
   {
-    sprintf(*this, "%s%c%s.XXXXXX", this->temporary_directory(), path_separator, PACKAGE_NAME);
+    sprintf(
+      this->buffer.data(),
+      "%s%c%s.XXXXXX",
+      this->temporary_directory(),
+      path_separator,
+      PACKAGE_NAME
+    );
+  }
+  operator char * ()
+  {
+    return this->buffer.data();
   }
 };
 
@@ -309,7 +320,7 @@ TemporaryDirectory::TemporaryDirectory() : Directory()
 {
 #if !WIN32
   TemporaryPathTemplate path_buffer;
-  if (mkdtemp(path_buffer) == NULL)
+  if (mkdtemp(path_buffer) == nullptr)
     throw_posix_error(static_cast<char*>(path_buffer));
 #else
   char base_path_buffer[PATH_MAX];
@@ -337,25 +348,26 @@ TemporaryDirectory::~TemporaryDirectory() throw ()
  * =========================
  */
 
-void File::open(const char* path, bool truncate)
+File::openmode File::get_default_open_mode()
 {
-  std::fstream::openmode mode = std::fstream::in | std::fstream::out | std::fstream::binary;
-  if (truncate)
-    mode |= std::fstream::trunc;
+  return std::fstream::trunc;
+}
+
+void File::open(const std::string &path, File::openmode mode)
+{
+  mode |=
+    std::fstream::in |
+    std::fstream::out |
+    std::fstream::binary;
   this->exceptions(std::ifstream::failbit | std::ifstream::badbit);
-  if (path == NULL)
-    this->std::fstream::open(this->name.c_str(), mode);
-  else
-  {
-    this->name = path;
-    this->std::fstream::open(path, mode);
-  }
+  this->name = path;
+  this->std::fstream::open(path.c_str(), mode);
   this->exceptions(std::ifstream::badbit);
 }
 
-File::File(const std::string &name)
+File::File(const std::string &path)
 {
-  this->open(name.c_str());
+  this->open(path, this->get_default_open_mode());
 }
 
 File::File(const Directory& directory, const std::string &name)
@@ -363,7 +375,7 @@ File::File(const Directory& directory, const std::string &name)
   std::ostringstream stream;
   this->base_name = name;
   stream << directory << path_separator << name;
-  this->open(stream.str().c_str());
+  this->open(stream.str(), this->get_default_open_mode());
 }
 
 File::streamoff File::size()
@@ -375,11 +387,11 @@ File::streamoff File::size()
   return result;
 }
 
-void File::reopen(bool truncate)
+void File::reopen(std::fstream::openmode mode)
 {
   if (this->is_open())
     this->close();
-  this->open(NULL, truncate);
+  this->open(this->name, mode);
 }
 
 const std::string& File::get_basename() const
@@ -424,7 +436,7 @@ void TemporaryFile::construct()
   if (GetTempFileName(base_path_buffer, PACKAGE_NAME, 0, path_buffer) == 0)
     throw_win32_error("GetTempFileName");
 #endif
-  this->open(path_buffer);
+  this->open(std::string(path_buffer), File::trunc);
 }
 
 TemporaryFile::TemporaryFile()
@@ -445,20 +457,9 @@ TemporaryFile::~TemporaryFile() throw ()
  * =========================
  */
 
-ExistingFile::ExistingFile(const std::string &name)
-: File()
+File::openmode ExistingFile::get_default_open_mode()
 {
-  this->name = name;
-  this->open(NULL, false);
-}
-
-ExistingFile::ExistingFile(const Directory& directory, const std::string &name)
-: File()
-{
-  std::ostringstream stream;
-  stream << directory << "/" << name;
-  this->name = stream.str();
-  this->open(NULL, false);
+  return File::openmode();
 }
 
 #if WIN32
@@ -473,8 +474,8 @@ Cwd::Cwd(const std::string &path)
   size_t size = 32;
   while (1)
   {
-    Array<char> buffer(size);
-    rc = getcwd(buffer, size) == NULL;
+    std::vector<char> buffer(size);
+    rc = getcwd(buffer.data(), size) == nullptr;
     if (rc != 0)
     {
       if (errno == ERANGE && size < SIZE_MAX / 2)
@@ -484,7 +485,7 @@ Cwd::Cwd(const std::string &path)
       }
       throw_posix_error("getcwd");
     }
-    this->previous_cwd = buffer;
+    this->previous_cwd = buffer.data();
     break;
   }
   rc = chdir(path.c_str());
@@ -516,7 +517,7 @@ Cwd::~Cwd()
 ProgramDir::ProgramDir()
 {
   char buffer[PATH_MAX];
-  size_t n = GetModuleFileName(NULL, buffer, sizeof buffer);
+  size_t n = GetModuleFileName(nullptr, buffer, sizeof buffer);
   if (n == 0)
     throw_win32_error("GetModuleFileName");
   if (n >= sizeof buffer)
@@ -565,10 +566,12 @@ bool isatty(const std::ostream &ostream)
 {
   if (&ostream == &std::cout)
     return isatty(STDOUT_FILENO);
+  else if (&ostream == &std::cerr || &ostream == &std::clog)
+    return isatty(STDERR_FILENO);
   else
   {
     /* Not implemented for streams other that ``std::cout``.
-     * See http://www.ginac.de/~kreckel/fileno/ for a more general
+     * See https://www.ginac.de/~kreckel/fileno/ for a more general
      * (although unportable, GCC-specific) solution.
      */
     throw std::invalid_argument("isatty(const std::ostream &)");
@@ -602,12 +605,12 @@ void split_path(const std::string &path, std::string &directory_name, std::strin
    * purposes.
    */
   size_t wlength, alength, length = path.length();
-  Array<wchar_t> wpath(length);
-  Array<char> apath(length);
+  std::vector<wchar_t> wpath(length);
+  std::vector<char> apath(length);
   wlength = MultiByteToWideChar(
     CP_ACP, 0,
     path.c_str(), length,
-    wpath, length
+    wpath.data(), length
   );
   if (wlength == 0)
     throw_win32_error("MultiByteToWideChar");
@@ -629,27 +632,27 @@ void split_path(const std::string &path, std::string &directory_name, std::strin
   }
   alength = WideCharToMultiByte(
     CP_ACP, 0,
-    wpath, l,
-    apath, length * 2,
-    NULL, NULL
+    wpath.data(), l,
+    apath.data(), length * 2,
+    nullptr, nullptr
   );
   if (l > 0)
   {
     if (alength == 0)
       throw_win32_error("WideCharToMultiByte");
-    directory_name = std::string(apath, alength);
+    directory_name = std::string(apath.data(), alength);
   }
   else
     directory_name = ".";
   alength = WideCharToMultiByte(
     CP_ACP, 0,
-    wpath + r, wlength - r,
-    apath, length * 2,
-    NULL, NULL
+    wpath.data() + r, wlength - r,
+    apath.data(), length * 2,
+    nullptr, nullptr
   );
   if (alength == 0 && r < wlength)
     throw_win32_error("WideCharToMultiByte");
-  file_name = std::string(apath, alength);
+  file_name = std::string(apath.data(), alength);
 #else
   /* POSIX-compliant ``basename()`` and ``dirname()`` would split ``/foo/bar/``
    * into ``/foo`` and ``bar``, instead of desired ``/foo/bar`` and an empty
@@ -657,14 +660,14 @@ void split_path(const std::string &path, std::string &directory_name, std::strin
    * appended to the split path.
    */
   {
-    Array<char> buffer(path.length() + 2);
-    sprintf(buffer, "%s!", path.c_str());
-    directory_name = ::dirname(buffer);
+    std::vector<char> buffer(path.length() + 2);
+    sprintf(buffer.data(), "%s!", path.c_str());
+    directory_name = ::dirname(buffer.data());
   }
   {
-    Array<char> buffer(path.length() + 2);
-    sprintf(buffer, "%s!", path.c_str());
-    file_name = ::basename(buffer);
+    std::vector<char> buffer(path.length() + 2);
+    sprintf(buffer.data(), "%s!", path.c_str());
+    file_name = ::basename(buffer.data());
     size_t length = file_name.length();
     assert(length > 0);
     assert(file_name[length - 1] == '!');
@@ -695,14 +698,14 @@ bool is_same_file(const std::string &path1, const std::string &path2)
   BY_HANDLE_FILE_INFORMATION info1, info2;
   HANDLE handle;
   int ok;
-  handle = CreateFile(path1.c_str(), FILE_READ_ATTRIBUTES, FILE_SHARE_READ, NULL, OPEN_EXISTING, 0, NULL);
+  handle = CreateFile(path1.c_str(), FILE_READ_ATTRIBUTES, FILE_SHARE_READ, nullptr, OPEN_EXISTING, 0, nullptr);
   if (handle == INVALID_HANDLE_VALUE)
     return false;
   ok = GetFileInformationByHandle(handle, &info1);
   CloseHandle(handle);
   if (!ok)
     return false;
-  handle = CreateFile(path2.c_str(), FILE_READ_ATTRIBUTES, FILE_SHARE_READ, NULL, OPEN_EXISTING, 0, NULL);
+  handle = CreateFile(path2.c_str(), FILE_READ_ATTRIBUTES, FILE_SHARE_READ, nullptr, OPEN_EXISTING, 0, nullptr);
   if (handle == INVALID_HANDLE_VALUE)
     return false;
   ok = GetFileInformationByHandle(handle, &info2);
@@ -728,7 +731,7 @@ bool is_same_file(const std::string &path1, const std::string &path2)
 #endif
 }
 
-void prevent_pop_out(void)
+void prevent_pop_out()
 {
 #if WIN32
   /* GetConsoleProcessList() function is not available for some systems (e.g.,
@@ -737,15 +740,15 @@ void prevent_pop_out(void)
   typedef DWORD (WINAPI *get_console_process_list_fn)(LPDWORD, DWORD);
   get_console_process_list_fn get_console_process_list;
   HMODULE dll = GetModuleHandle("kernel32");
-  if (dll == NULL)
+  if (dll == nullptr)
     return;
   get_console_process_list = (get_console_process_list_fn) GetProcAddress(dll, "GetConsoleProcessList");
-  if (get_console_process_list != NULL)
+  if (get_console_process_list != nullptr)
   {
     unsigned long pid, rc;
     rc = get_console_process_list(&pid, 1);
     if (rc == 1)
-      MessageBox(NULL, _("pdf2djvu is intended to be run from the command prompt."), PACKAGE_NAME, MB_OK | MB_ICONINFORMATION);
+      MessageBox(nullptr, _("pdf2djvu is intended to be run from the command prompt."), PACKAGE_NAME, MB_OK | MB_ICONINFORMATION);
   }
 #endif
 }
